/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifndef ZzAudioUtilities_h
#include "ZzAudioUtilities.h"
#endif //ZzAudioUtilities_h

void ZzAudioFillWithStereoFloatSin128(float* buffer, SInt32 nbFrames, SInt32 phase)
{
	// a cycle is 128 frames
	// thus, 1 frame is 1/128 of 2*ArboPi which is ArboPi/64 = 0.0490873852123
	phase = phase % 128;
	double doublePhase = phase*0.0490873852123;
	for(SInt32 frame = 0; frame < nbFrames; frame++)
	{
		float sinVal = (float)sin(doublePhase);
		*buffer++ = sinVal;
		*buffer++ = sinVal;
		doublePhase += 0.0490873852123;
	}
}

void ZzAudioFillWithStereoSInt16Sin128(SInt16* buffer, SInt32 nbFrames, SInt32 phase)
{
	// a cycle is 128 frames
	// thus, 1 frame is 1/128 of 2*ArboPi which is ArboPi/64 = 0.0490873852123
	phase = phase % 128;
	double doublePhase = phase*0.0490873852123;
	for(SInt32 sample = 0; sample < nbFrames*2; sample++)
	{
		SInt16 sinVal = (SInt16)(32767.0*(float)sin(doublePhase));
		*buffer++ = sinVal;
		*buffer++ = sinVal;
		doublePhase += 0.0490873852123;
	}
}

float* ZzAudioGetRingBufferAddress(float* RB, SInt32 RBNbBits, SInt32 nbChannels, SInt64 posInRB)
{
	UInt32 frameIndexInRB = posInRB & ((1 << RBNbBits) - 1);
	SInt32 sampleIndexInRB = frameIndexInRB * nbChannels;
	float* addr = RB + sampleIndexInRB;
	return addr;
}

#ifdef __APPLE__

SInt32 ZzAudioCAGetNbAvailableOutputStreamsInDevice(AudioDeviceID inDeviceID)
{
	UInt32							theSize;
	OSStatus						theStatus;
	UInt32							theNumberOutputChannels;
	AudioBufferList*				theBufferList;
	UInt32							theIndex;

	//	output stream configuration size
	theNumberOutputChannels = 0;
	theSize = 0;
	theStatus = AudioDeviceGetPropertyInfo(inDeviceID, 0, 0, kAudioDevicePropertyStreamConfiguration, &theSize, NULL);
	//ZzAssert(theStatus == noErr);
	//MessageIfOSError(theStatus, "  Couldn't get the output stream configuration size");
	// normally theStatus should be noErr but in case this is different, we assume 0 output channels
	
	if((theStatus == 0) && (theSize != 0))
	{
		//	allocate the space
		theBufferList = (AudioBufferList*)malloc(theSize);
		ZzAssert(theBufferList != NULL);
		if(theBufferList != NULL)
		{
			//	get the output stream configuration
			theStatus = AudioDeviceGetProperty(inDeviceID, 0, 0, kAudioDevicePropertyStreamConfiguration, &theSize, theBufferList);
			ZzAssert(theStatus == noErr);
			//MessageIfOSError(theStatus, "  Couldn't get the output buffer list");
			if(theStatus == 0)
			{
				//	print the output stream configuration
				//PrintLine("  Output stream configuration");
				//DisplayAudioBufferList(theBufferList);
				
				//	get the total number of channels for later
				for(theIndex = 0; theIndex < theBufferList->mNumberBuffers; ++theIndex)
				{
					theNumberOutputChannels += theBufferList->mBuffers[theIndex].mNumberChannels;
				}
			}

			//	free the list
			free(theBufferList);
		}
	}
	else
	{
		//PrintLine("  No output streams available");
	}
	return theNumberOutputChannels;
}

SInt32 ZzAudioCAGetNbAvailableInputStreamsInDevice(AudioDeviceID inDeviceID)
{
	UInt32							theSize;
	OSStatus						theStatus;
	UInt32							theNumberInputChannels;
	AudioBufferList*				theBufferList;
	UInt32							theIndex;

	//	input stream configuration size
	theNumberInputChannels = 0;
	theSize = 0;
	theStatus = AudioDeviceGetPropertyInfo(inDeviceID, 0, 1, kAudioDevicePropertyStreamConfiguration, &theSize, NULL);
	//MessageIfOSError(theStatus, "  Couldn't get the input stream configuration size");
	
	if((theStatus == 0) && (theSize != 0))
	{
		//	allocate the space
		theBufferList = (AudioBufferList*)malloc(theSize);
		ZzAssert(theBufferList != NULL);
		//MessageIfNULL(theBufferList, "  Couldn't allocate input buffer list");
		if(theBufferList != NULL)
		{
			//	get the input stream configuration
			theStatus = AudioDeviceGetProperty(inDeviceID, 0, 1, kAudioDevicePropertyStreamConfiguration, &theSize, theBufferList);
			ZzAssert(theStatus == noErr);
			//MessageIfOSError(theStatus, "  Couldn't get the input buffer list");
			if(theStatus == 0)
			{
				//	print the input stream configuration
//				PrintLine("  Input stream configuration");
//				DisplayAudioBufferList(theBufferList);
				
				//	get the total number of channels for later
				for(theIndex = 0; theIndex < theBufferList->mNumberBuffers; ++theIndex)
				{
					theNumberInputChannels += theBufferList->mBuffers[theIndex].mNumberChannels;
				}
			}
			//	free the list
			free(theBufferList);
		}
	}
	else
	{
		//PrintLine("  No input streams available");
	}
	return theNumberInputChannels;
}

#endif //#ifdef __APPLE__

void ZzAudioMixStereoWithStrides(
	float* 			dst, 
	SInt32 			dstLeftChannelIndex,
	SInt32			dstNbChannels,
	const float*	src, 
	SInt32 			srcLeftChannelIndex, 
	SInt32			srcNbChannels,
	SInt32 			nbFrames )
{
//	printf("ZzAudioMixStereoWithStrides reading from %p to %p (%d floats) and writing from %p to %p (%d floats)\n",dst,dst + dstNbChannels*nbFrames,(int)dstNbChannels*nbFrames,src,src+srcNbChannels*nbFrames,(int)srcNbChannels*nbFrames);
	// let's patch the strides for sequential incrementation (see code)
	dst += dstLeftChannelIndex;
	src += srcLeftChannelIndex;
		
	SInt32 srcStride = srcNbChannels - 1;  // stride from channel R of frame N to channel L of frame N+1
	SInt32 dstStride = dstNbChannels - 1;  // stride from channel R of frame N to channel L of frame N+1
	
	for(SInt32 i = 0; i < nbFrames; i++)
	{
		*dst += *src;
		dst++;
		src++;
		*dst += *src;
		dst += dstStride;
		src += srcStride;
	}
}

